<?php

/*
+---------------------------------------------------------------------------+
| Max Media Manager v0.2                                                    |
| =================                                                         |
|                                                                           |
| Copyright (c) 2003-2004 Awarez Ltd.                                       |
| For contact details, see: http://www.awarez.net/                          |
|                                                                           |
| This program is free software; you can redistribute it and/or modify      |
| it under the terms of the GNU General Public License as published by      |
| the Free Software Foundation; either version 2 of the License, or         |
| (at your option) any later version.                                       |
|                                                                           |
| This program is distributed in the hope that it will be useful,           |
| but WITHOUT ANY WARRANTY; without even the implied warranty of            |
| MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             |
| GNU General Public License for more details.                              |
|                                                                           |
| You should have received a copy of the GNU General Public License         |
| along with this program; if not, write to the Free Software               |
| Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA |
+---------------------------------------------------------------------------+
$Id: StatisticsAdServerSplit.php 1013 2004-11-26 11:43:27Z andrew $
*/

/**
 * @package    MaxDal
 * @subpackage Statistics
 * @author     Andrew Hill <andrew@awarez.net>
 */

require_once maintenancePath . '/maintenance-common.php';
require_once maintenancePath . '/libraries/max/Dal/StatisticsAdServer.php';
require_once maintenancePath . '/libraries/pear/Date.php';
 
/**
 * The data access layer code for summarising raw data into statistics, for 
 * the AdServer module, when split tables are in use
 */
class Dal_Statistics_AdServer_Split extends Dal_Statistics_AdServer
{

    /**
     * The constructor method.
     *
     * @uses Dal_Statistics_AdServer::Dal_Statistics_AdServer()
     */
    function Dal_Statistics_AdServer_Split()
    {
        parent::Dal_Statistics_AdServer();
    }
    
    /**
     * The singleton method to create or return a single instance of the class.
     *
     * @static
     * @return Dal_Statistics_AdServer_Split The returned Dal_Statistics_AdServer_Split object.
     */
    function &singleton() {
        static $instance;
        if (!isset($instance)) {
            $instance = new Dal_Statistics_AdServer_Split();
        }
        return $instance;
    }
    
    /**
     * A method to find the last time that maintenance statistics was run.
     *
     * @param integer $type The update type that occurred - that is, 0 if
     *                      the update was done on the basis of the 
     *                      operation interval, or 1 if the update was
     *                      done on the basis of the hour.
     * @return Date A Date representing the date up to which the statistics
     *              have been summarised, for the specified update type, or
     *              the appropriate date based on raw data if maintenance
     *              statistics has never been run for the Max module before.
     *              Returns null if no raw data is available.
     */
    function getMaintenanceStatisticsLastRunInfo($type)
    {
        $table = $this->conf['table']['data_raw_ad_impression'] . '_' . date('Ymd');
        return $this->_getMaintenanceStatisticsLastRunInfo($type, $table);
    }
    
    /**
     * A method for summarising impressions into a temporary table.
     *
     * @param Date $start The start date/time to summarise from.
     * @param Date $end The end date/time to summarise to.
     * @return integer The number of conversion rows summarised.
     */
    function summariseImpressions($start, $end)
    {
        // Check the start and end dates
        if (!$this->_checkIntervalDates($start, $end)) {
            return 0;
        }
        // Get the start and end dates of the operation interval ID
        list($operationIntervalStart, $operationIntervalEnd) =
            Maintenance::convertDateToOperationIntervalStartAndEndDates($start);
        // How many days does the operation interval span?
        $days = Date_Calc::dateDiff($operationIntervalStart->getDay(),
                                    $operationIntervalStart->getMonth(),
                                    $operationIntervalStart->getYear(),
                                    $operationIntervalEnd->getDay(),
                                    $operationIntervalEnd->getMonth(),
                                    $operationIntervalEnd->getYear());
        // Get the operation interval ID
        $operationIntervalID = Maintenance::convertDateToOperationIntervalID($operationIntervalStart);
        // Create the temporary summary table
        $this->tempTables->createTable('tmp_ad_impression');
        // Summarise the impressions
        $rows = 0;
        $currentDate = $operationIntervalStart;
        for ($counter = 0; $counter <= $days; $counter++) {
            // Set the appropriate data_raw_ad_impression table
            $impressionTable = $this->conf['table']['data_raw_ad_impression'] . '_' .
                               $currentDate->format('%Y%m%d');
            $query = "
                INSERT INTO
                    tmp_ad_impression
                    (
                        day,
                        hour,
                        operation_interval,
                        operation_interval_id,
                        interval_start,
                        interval_end,
                        ad_id,
                        creative_id,
                        zone_id,
                        impressions
                    )
                SELECT
                    DATE_FORMAT(drai.date_time, '%Y-%m-%d') AS day,
                    DATE_FORMAT(drai.date_time, '%k') AS hour,
                    {$this->conf['maintenance']['operation_interval']} AS operation_interval,
                    $operationIntervalID AS operation_interval_id,
                    '".$operationIntervalStart->format('%Y-%m-%d %H:%M:%S')."' AS interval_start,
                    '".$operationIntervalEnd->format('%Y-%m-%d %H:%M:%S')."' AS interval_end,
                    drai.ad_id AS ad_id,
                    drai.creative_id AS creative_id,
                    drai.zone_id AS zone_id,
                    COUNT(*) AS impressions
                FROM
                    $impressionTable AS drai
                WHERE
                    drai.date_time >= ".$start->format('%Y%m%d%H%M%S')."
                    AND drai.date_time <= ".$end->format('%Y%m%d%H%M%S')."
                GROUP BY
                    day, hour, ad_id, creative_id, zone_id";
            debug("Summarising ad impressions from the $impressionTable table", PEAR_LOG_DEBUG);
            $errors = array(1146);   // Permit the table does not exist error
            $errorID = $this->executeIgnoreErrors($query, $result, $errors);
            if ($errorID == 0) {
                $rows += mysql_affected_rows($this->dbConnection);
            }
            // Update the split data_raw_ad_impression table being used
            $currentDate = $currentDate->getNextDay();
        }
        return $rows;
    }

    /**
     * A method for summarising clicks into a temporary table.
     *
     * @param Date $start The start date/time to summarise from.
     * @param Date $end The end date/time to summarise to.
     * @return integer The number of conversion rows summarised.
     */    
    function summariseClicks($start, $end)
    {
        // Check the start and end dates
        if (!$this->_checkIntervalDates($start, $end)) {
            return 0;
        }
        // Get the start and end dates of the operation interval ID
        list($operationIntervalStart, $operationIntervalEnd) =
            Maintenance::convertDateToOperationIntervalStartAndEndDates($start);
        // How many days does the operation interval span?
        $days = Date_Calc::dateDiff($operationIntervalStart->getDay(),
                                    $operationIntervalStart->getMonth(),
                                    $operationIntervalStart->getYear(),
                                    $operationIntervalEnd->getDay(),
                                    $operationIntervalEnd->getMonth(),
                                    $operationIntervalEnd->getYear());
        // Get the operation interval ID
        $operationIntervalID = Maintenance::convertDateToOperationIntervalID($operationIntervalStart);
        // Create the temporary summary table
        $this->tempTables->createTable('tmp_ad_click');
        // Summarise the clicks
        $rows = 0;
        $currentDate = $operationIntervalStart;
        for ($counter = 0; $counter <= $days; $counter++) {
            // Set the appropriate data_raw_ad_click table
            $clickTable = $this->conf['table']['data_raw_ad_click'] . '_' . $currentDate->format('%Y%m%d');
            $query = "
                INSERT INTO
                    tmp_ad_click
                    (
                        day,
                        hour,
                        operation_interval,
                        operation_interval_id,
                        interval_start,
                        interval_end,
                        ad_id,
                        creative_id,
                        zone_id,
                        clicks
                    )
                SELECT
                    DATE_FORMAT(drac.date_time, '%Y-%m-%d') AS day,
                    DATE_FORMAT(drac.date_time, '%k') AS hour,
                    {$this->conf['maintenance']['operation_interval']} AS operation_interval,
                    $operationIntervalID AS operation_interval_id,
                    '".$operationIntervalStart->format('%Y-%m-%d %H:%M:%S')."' AS interval_start,
                    '".$operationIntervalEnd->format('%Y-%m-%d %H:%M:%S')."' AS interval_end,
                    drac.ad_id AS ad_id,
                    drac.creative_id AS creative_id,
                    drac.zone_id AS zone_id,
                    COUNT(*) AS clicks
                FROM
                    $clickTable AS drac
                WHERE
                    drac.date_time >= ".$start->format('%Y%m%d%H%M%S')."
                    AND drac.date_time <= ".$end->format('%Y%m%d%H%M%S')."
                GROUP BY
                    day, hour, ad_id, creative_id, zone_id";
            debug("Summarising ad clicks from the $clickTable table", PEAR_LOG_DEBUG);
            $errors = array(1146);   // Permit the table does not exist error
            $errorID = $this->executeIgnoreErrors($query, $result, $errors);
            if ($errorID == 0) {
                $rows += mysql_affected_rows($this->dbConnection);
            }
            // Update the split data_raw_ad_click table being used
            $currentDate = $currentDate->getNextDay();
        }
        return $rows;
    }
    
    /**
     * A method for summarising connections into a temporary table.
     *
     * @param Date $start The start date/time to summarise from.
     * @param Date $end The end date/time to summarise to.
     * @return integer The number of connections summarised.
     */
    function summariseConnections($start, $end)
    { 
        if (!$this->conf['modules']['Tracker']) {
            return 0;
        }
        // Check the start and end dates
        if (!$this->_checkIntervalDates($start, $end)) {
            return 0;
        }
        // Get the start and end dates of the operation interval ID
        list($operationIntervalStart, $operationIntervalEnd) =
            Maintenance::convertDateToOperationIntervalStartAndEndDates($start);
        // How many days does the operation interval span?
        $days = Date_Calc::dateDiff($operationIntervalStart->getDay(),
                                    $operationIntervalStart->getMonth(),
                                    $operationIntervalStart->getYear(),
                                    $operationIntervalEnd->getDay(),
                                    $operationIntervalEnd->getMonth(),
                                    $operationIntervalEnd->getYear());
        // Get the operation interval ID
        $operationIntervalID = Maintenance::convertDateToOperationIntervalID($operationIntervalStart);
        // Get the maximum impression and click connection windows
        list($impressionWindow, $clickWindow) = Maintenance::maxConnectionWindows();
        // Create the tmp_ad_connection table
        
        $this->tempTables->createTable('tmp_ad_connection');       
        // Insert tracker impressions that may connect to ad impressions into
        // the temporary tmp_tracker_impression_ad_impression_connection table
        $currentDate = $operationIntervalStart;
        for ($counter = 0; $counter <= $days; $counter++) {
            // Create the tmp_tracker_impression_ad_impression_connection table
            $this->tempTables->createTable('tmp_tracker_impression_ad_impression_connection');
            // Set the appropriate data_raw_tracker_impression table
            $trackerImpressionTable = $this->conf['table']['data_raw_tracker_impression'] . '_' .
                                      $currentDate->format('%Y%m%d');
            $query = "
                INSERT INTO
                    tmp_tracker_impression_ad_impression_connection
                    (
                        server_raw_tracker_impression_id,
                        server_raw_ip,
                        tracker_id,
                        campaign_id,
                        operation_interval,
                        operation_interval_id,
                        interval_start,
                        interval_end,
                        viewer_id,
                        date_time,
                        connection_type,
                        connection_window,
                        conversion
                    )
                SELECT
                    drti.server_raw_tracker_impression_id AS server_raw_tracker_impression_id,
                    drti.server_raw_ip AS server_raw_ip,
                    drti.tracker_id AS tracker_id,
                    ct.campaignid AS campaign_id,
                    {$this->conf['maintenance']['operation_interval']} AS operation_interval,
                    $operationIntervalID AS operation_interval_id,
                    '".$operationIntervalStart->format('%Y-%m-%d %H:%M:%S')."' AS interval_start,
                    '".$operationIntervalEnd->format('%Y-%m-%d %H:%M:%S')."' AS interval_end,
                    drti.viewer_id AS viewer_id,
                    drti.date_time AS date_time,
                    '0' AS connection_type,
                    ct.viewwindow AS connection_window,
                    ct.logstats AS conversion
                FROM
                    $trackerImpressionTable AS drti,        
                    {$this->conf['table']['campaigns_trackers']} AS ct
                WHERE        
                    drti.tracker_id = ct.trackerid
                    AND drti.viewer_id != ''
                    AND drti.date_time >= ".$start->format('%Y%m%d%H%M%S')."
                    AND drti.date_time <= ".$end->format('%Y%m%d%H%M%S')."
                    AND ct.viewwindow > 0";
            debug('Selecting tracker impressions that may connect to ad impressions into the ' .
                  'tmp_tracker_impression_ad_impression_connection temporary table', PEAR_LOG_DEBUG);
            $errors = array(1146);   // Permit the table does not exist error
            $errorID = $this->executeIgnoreErrors($query, $result, $errors);
            if (($errorID == 0) && (mysql_affected_rows($this->dbConnection) > 0)) {
                // Connect the tracker impressions with ad impressions, where possible, by looking
                // over the necessary past data_raw_ad_impression tables, as appropriate for the
                // maximum impression connection window
                $endImpressionDate = $currentDate;
                $startImpressionDate = $endImpressionDate;
                $startImpressionDate->subtractSeconds((int) $impressionWindow); // Cast to int as Date class
                                                                                // doesn't correctly deal with
                                                                                // integers as strings...
                // How many days does the impression connection window span?
                $impressionDays = Date_Calc::dateDiff($startImpressionDate->getDay(),
                                                      $startImpressionDate->getMonth(),
                                                      $startImpressionDate->getYear(),
                                                      $endImpressionDate->getDay(),
                                                      $endImpressionDate->getMonth(),
                                                      $endImpressionDate->getYear());
                $currentImpressionDate = $startImpressionDate;
                for ($impressionCounter = 0; $impressionCounter <= $impressionDays; $impressionCounter++) {
                    // Set the appropriate data_raw_ad_impression table
                    $adImpressionTable = $this->conf['table']['data_raw_ad_impression'] . '_' .
                                         $currentImpressionDate->format('%Y%m%d');
                    $query = "
                        INSERT INTO
                            tmp_ad_connection
                            (
                                server_raw_tracker_impression_id,
                                server_raw_ip,
                                date_time,
                                operation_interval,
                                operation_interval_id,
                                interval_start,
                                interval_end,
                                connection_viewer_id,
                                connection_viewer_session_id,
                                connection_date_time,
                                connection_ad_id,
                                connection_creative_id,
                                connection_zone_id,
                                connection_channel,
                                connection_language,
                                connection_ip_address,
                                connection_host_name,
                                connection_country,
                                connection_https,
                                connection_domain,
                                connection_page,
                                connection_query,
                                connection_referer,
                                connection_search_term,
                                connection_user_agent,
                                connection_os,
                                connection_browser,
                                connection_type,
                                connection_window,
                                conversion
                            )
                        SELECT
                            ttiaic.server_raw_tracker_impression_id AS server_raw_tracker_impression_id,
                            ttiaic.server_raw_ip AS server_raw_ip,
                            ttiaic.date_time AS date_time,
                            ttiaic.operation_interval AS operation_interval,
                            ttiaic.operation_interval_id AS operation_interval_id,
                            ttiaic.interval_start AS interval_start,
                            ttiaic.interval_end AS interval_end,
                            drai.viewer_id AS connection_viewer_id,
                            drai.viewer_session_id AS connection_viewer_session_id,
                            drai.date_time AS connection_date_time,
                            drai.ad_id AS connection_ad_id,
                            drai.creative_id AS connection_creative_id,
                            drai.zone_id AS connection_zone_id,
                            drai.channel AS connection_channel,
                            drai.language AS connection_language,
                            drai.ip_address AS connection_ip_address,
                            drai.host_name AS connection_host_name,
                            drai.country AS connection_country,
                            drai.https AS connection_https,
                            drai.domain AS connection_domain,
                            drai.page AS connection_page,
                            drai.query AS connection_query,
                            drai.referer AS connection_referer,
                            drai.search_term AS connection_search_term,
                            drai.user_agent AS connection_user_agent,
                            drai.os AS connection_os,
                            drai.browser AS connection_browser,
                            ttiaic.connection_type AS connection_type,
                            ttiaic.connection_window AS connection_window,
                            ttiaic.conversion AS conversion
                        FROM
                            tmp_tracker_impression_ad_impression_connection AS ttiaic,
                            $adImpressionTable AS drai,
                            {$this->conf['table']['banners']} AS b,
                            {$this->conf['table']['campaigns_trackers']} AS ct
                        WHERE
                            ttiaic.viewer_id = drai.viewer_id
                            AND drai.ad_id = b.bannerid
                            AND b.campaignid = ct.campaignid
                            AND ct.trackerid = ttiaic.tracker_id
                            AND ttiaic.campaign_id = ct.campaignid
                            AND ttiaic.date_time < DATE_ADD(drai.date_time, INTERVAL ttiaic.connection_window SECOND)
                            AND ttiaic.date_time >= drai.date_time";
                    debug("Connecting tracker impressions with ad impressions from the $adImpressionTable " .
                           'table, where possible', PEAR_LOG_DEBUG);
                    $errors = array(1146);   // Permit the table does not exist error
                    $errorID = $this->executeIgnoreErrors($query, $result, $errors);
                    if ($errorID == 0) {
                        $rows += mysql_affected_rows();
                    }
                    // Update the split data_raw_ad_impression table being used
                    $currentImpressionDate = $currentImpressionDate->getNextDay();
                }
            }
            // Drop the temporary table for storing tracker impressions that may connect to ad impressions
            $this->tempTables->dropTempTable('tmp_tracker_impression_ad_impression_connection');
            // Update the split data_raw_ad_impression table being used
            $currentDate = $currentDate->getNextDay();
        }
        // Insert tracker impressions that may connect to ad clicks into
        // the temporary tmp_tracker_impression_ad_click_connection table
        $currentDate = $operationIntervalStart;
        for ($counter = 0; $counter <= $days; $counter++) {
            // Create the tmp_tracker_impression_ad_click_connection table
            $this->tempTables->createTable('tmp_tracker_impression_ad_click_connection');
            // Set the appropriate data_raw_tracker_impression table
            $trackerImpressionTable = $this->conf['table']['data_raw_tracker_impression'] . '_' .
                                      $currentDate->format('%Y%m%d');
            $query = "
                INSERT INTO
                    tmp_tracker_impression_ad_click_connection
                    (
                        server_raw_tracker_impression_id,
                        server_raw_ip,
                        tracker_id,
                        campaign_id,
                        operation_interval,
                        operation_interval_id,
                        interval_start,
                        interval_end,
                        viewer_id,
                        date_time,
                        connection_type,
                        connection_window,
                        conversion
                    )
                SELECT
                    drti.server_raw_tracker_impression_id AS server_raw_tracker_impression_id,
                    drti.server_raw_ip AS server_raw_ip,
                    drti.tracker_id AS tracker_id,
                    ct.campaignid AS campaign_id,
                    {$this->conf['maintenance']['operation_interval']} AS operation_interval,
                    $operationIntervalID AS operation_interval_id,
                    '".$operationIntervalStart->format('%Y-%m-%d %H:%M:%S')."' AS interval_start,
                    '".$operationIntervalEnd->format('%Y-%m-%d %H:%M:%S')."' AS interval_end,
                    drti.viewer_id AS viewer_id,
                    drti.date_time AS date_time,
                    '1' AS connection_type,
                    ct.clickwindow AS connection_window,
                    ct.logstats AS conversion
                FROM
                    $trackerImpressionTable AS drti,        
                    {$this->conf['table']['campaigns_trackers']} AS ct
                WHERE        
                    drti.tracker_id = ct.trackerid
                    AND drti.viewer_id != ''
                    AND drti.date_time >= ".$start->format('%Y%m%d%H%M%S')."
                    AND drti.date_time <= ".$end->format('%Y%m%d%H%M%S')."
                    AND ct.clickwindow > 0";
            $errors = array(1146);   // Permit the table does not exist error
            $errorID = $this->executeIgnoreErrors($query, $result, $errors);
            if (($errorID == 0) && (mysql_affected_rows($this->dbConnection) > 0)) {
                // Connect the tracker impressions with ad clicks, where possible, by looking
                // over the necessary past data_raw_ad_click tables, as appropriate for the
                // maximum click connection window
                $endClickDate = $currentDate;
                $startClickDate = $endClickDate;
                $startClickDate->subtractSeconds((int) $clickWindow); // Cast to int as Date class
                                                                      // doesn't correctly deal with
                                                                      // integers as strings...
                // How many days does the impression connection window span?
                $clickDays = Date_Calc::dateDiff($startClickDate->getDay(),
                                                 $startClickDate->getMonth(),
                                                 $startClickDate->getYear(),
                                                 $endClickDate->getDay(),
                                                 $endClickDate->getMonth(),
                                                 $endClickDate->getYear());
                $currentClickDate = $startClickDate;
                for ($clickCounter = 0; $clickCounter <= $clickDays; $clickCounter++) {
                    // Set the appropriate data_raw_ad_click table
                    $adClickTable = $this->conf['table']['data_raw_ad_click'] . '_' .
                                    $currentClickDate->format('%Y%m%d');
                    $query = "
                        INSERT INTO
                            tmp_ad_connection
                            (
                                server_raw_tracker_impression_id,
                                server_raw_ip,
                                date_time,
                                operation_interval,
                                operation_interval_id,
                                interval_start,
                                interval_end,
                                connection_viewer_id,
                                connection_viewer_session_id,
                                connection_date_time,
                                connection_ad_id,
                                connection_creative_id,
                                connection_zone_id,
                                connection_channel,
                                connection_language,
                                connection_ip_address,
                                connection_host_name,
                                connection_country,
                                connection_https,
                                connection_domain,
                                connection_page,
                                connection_query,
                                connection_referer,
                                connection_search_term,
                                connection_user_agent,
                                connection_os,
                                connection_browser,
                                connection_type,
                                connection_window,
                                conversion
                            )
                        SELECT
                            ttiacc.server_raw_tracker_impression_id AS server_raw_tracker_impression_id,
                            ttiacc.server_raw_ip AS server_raw_ip,
                            ttiacc.date_time AS date_time,
                            ttiacc.operation_interval AS operation_interval,
                            ttiacc.operation_interval_id AS operation_interval_id,
                            ttiacc.interval_start AS interval_start,
                            ttiacc.interval_end AS interval_end,
                            drai.viewer_id AS connection_viewer_id,
                            drai.viewer_session_id AS connection_viewer_session_id,
                            drai.date_time AS connection_date_time,
                            drai.ad_id AS connection_ad_id,
                            drai.creative_id AS connection_creative_id,
                            drai.zone_id AS connection_zone_id,
                            drai.channel AS connection_channel,
                            drai.language AS connection_language,
                            drai.ip_address AS connection_ip_address,
                            drai.host_name AS connection_host_name,
                            drai.country AS connection_country,
                            drai.https AS connection_https,
                            drai.domain AS connection_domain,
                            drai.page AS connection_page,
                            drai.query AS connection_query,
                            drai.referer AS connection_referer,
                            drai.search_term AS connection_search_term,
                            drai.user_agent AS connection_user_agent,
                            drai.os AS connection_os,
                            drai.browser AS connection_browser,
                            ttiacc.connection_type AS connection_type,
                            ttiacc.connection_window AS connection_window,
                            ttiacc.conversion AS conversion
                        FROM
                            tmp_tracker_impression_ad_click_connection AS ttiacc,
                            $adClickTable AS drai,
                            {$this->conf['table']['banners']} AS b,
                            {$this->conf['table']['campaigns_trackers']} AS ct
                        WHERE
                            ttiacc.viewer_id = drai.viewer_id
                            AND drai.ad_id = b.bannerid
                            AND b.campaignid = ct.campaignid
                            AND ct.trackerid = ttiacc.tracker_id
                            AND ttiacc.campaign_id = ct.campaignid
                            AND ttiacc.date_time < DATE_ADD(drai.date_time, INTERVAL ttiacc.connection_window SECOND)
                            AND ttiacc.date_time >= drai.date_time";
                    debug("Connecting tracker impressions with ad clicks from the $adClickTable " .
                           'table, where possible', PEAR_LOG_DEBUG);
                    $errors = array(1146);   // Permit the table does not exist error
                    $errorID = $this->executeIgnoreErrors($query, $result, $errors);
                    if ($errorID == 0) {
                        $rows += mysql_affected_rows();
                    }
                    // Update the split data_raw_ad_click table being used
                    $currentClickDate = $currentClickDate->getNextDay();
                }
            }
            // Drop the temporary table for storing tracker impressions that may connect to ad clicks
            $this->tempTables->dropTempTable('tmp_tracker_impression_ad_click_connection');
            // Update the split data_raw_ad_impression table being used
            $currentDate = $currentDate->getNextDay();
        }        
        return $rows;
    }
    
    /**
     * A private method to save the tacker impression/ad impression and
     * tracker impression/ad click connections that have been marked as
     * "latest", and also save the associated tracker impression 
     * variable values.
     *
     * @private
     * @param Date $start The start date/time to save from.
     * @param Date $end The end date/time to save to.
     */
    function _saveConnectionsAndVariableValues($start, $end)
    {
        // How many days does the operation interval span?
        $days = Date_Calc::dateDiff($start->getDay(), $start->getMonth(), $start->getYear(),
                                    $end->getDay(),   $end->getMonth(),   $end->getYear());
        $outerTable = $this->conf['table']['data_intermediate_ad_connection'];
        $innterTable = $this->conf['table']['data_intermediate_ad_variable_value'];
        // Save the connections
        $outerDate = $start;
        for ($counter = 0; $counter <= $days; $counter++) {
            // Set the appropriate data_raw_tracker_impression table
            $trackerImpressionTable = $this->conf['table']['data_raw_tracker_impression'] . '_' .
                                      $outerDate->format('%Y%m%d');
            $query = "
                INSERT INTO
                    $outerTable
                    (
                        server_raw_tracker_impression_id,
                        server_raw_ip,
                        viewer_id,
                        viewer_session_id,
                        tracker_date_time,
                        connection_date_time,
                        tracker_id,
                        ad_id,
                        creative_id,
                        zone_id,
                        tracker_channel,
                        connection_channel,
                        tracker_language,
                        connection_language,
                        tracker_ip_address,
                        connection_ip_address,
                        tracker_host_name,
                        connection_host_name,
                        tracker_country,
                        connection_country,
                        tracker_https,
                        connection_https,
                        tracker_domain,
                        connection_domain,
                        tracker_page,
                        connection_page,
                        tracker_query,
                        connection_query,
                        tracker_referer,
                        connection_referer,
                        tracker_search_term,
                        connection_search_term,
                        tracker_user_agent,
                        connection_user_agent,
                        tracker_os,
                        connection_os,
                        tracker_browser,
                        connection_browser,
                        connection_type,
                        connection_window,
                        conversion
                    )
                SELECT
                    drti.server_raw_tracker_impression_id AS server_raw_tracker_impression_id,
                    drti.server_raw_ip AS server_raw_ip,
                    drti.viewer_id AS viewer_id,
                    drti.viewer_session_id AS viewer_session_id,
                    drti.date_time AS tracker_date_time,
                    tac.connection_date_time AS connection_date_time,
                    drti.tracker_id AS tracker_id,
                    tac.connection_ad_id AS ad_id,
                    tac.connection_creative_id AS creative_id,
                    tac.connection_zone_id AS zone_id,
                    drti.channel AS tracker_channel,
                    tac.connection_channel AS connection_channel,
                    drti.language AS tracker_language,
                    tac.connection_language AS connection_language,
                    drti.ip_address AS tracker_ip_address,
                    tac.connection_ip_address AS connection_ip_address,
                    drti.host_name AS tracker_host_name,
                    tac.connection_host_name AS connection_host_name,
                    drti.country AS tracker_country,
                    tac.connection_country AS connection_country,
                    drti.https AS tracker_https,
                    tac.connection_https AS connection_https,
                    drti.domain AS tracker_domain,
                    tac.connection_domain AS connection_domain,
                    drti.page AS tracker_page,
                    tac.connection_page AS connection_page,
                    drti.query AS tracker_query,
                    tac.connection_query AS connection_query,
                    drti.referer AS tracker_referer,
                    tac.connection_referer AS connection_referer,
                    drti.search_term AS tracker_search_term,
                    tac.connection_search_term AS connection_search_term,
                    drti.user_agent AS tracker_user_agent,
                    tac.connection_user_agent AS connection_user_agent,
                    drti.os AS tracker_os,
                    tac.connection_os AS connection_os,
                    drti.browser AS tracker_browser,
                    tac.connection_browser AS connection_browser,
                    tac.connection_type AS connection_type,
                    tac.connection_window AS connection_window,
                    tac.conversion AS conversion
                FROM
                    tmp_ad_connection AS tac,
                    $trackerImpressionTable AS drti
                WHERE
                    tac.server_raw_tracker_impression_id = drti.server_raw_tracker_impression_id
                    AND tac.server_raw_ip = drti.server_raw_ip
                    AND tac.latest = 1";
            debug("Inserting the connections into the $outerTable table", PEAR_LOG_DEBUG);
            $this->execute($query, $result);
            // Save any variable values associated with the above connections
            $innerDate = $outerDate;
            for ($innerCounter = 0; $innterCounter < 2; $innterCounter++) {
                // Set the appropriate data_raw_tracker_variable_value table
                $trackerVariableValueTable = $this->conf['table']['data_raw_tracker_variable_value'] . '_' .
                                             $innerDate->format('%Y%m%d');
                $query = "
                    INSERT INTO
                        $innterTable
                        (
                            data_intermediate_ad_connection_id,
                            tracker_variable_id,
                            value
                        )
                    SELECT
                        diac.data_intermediate_ad_connection_id AS data_intermediate_ad_connection_id,        
                        drtvv.tracker_variable_id AS tracker_variable_id,        
                        drtvv.value AS value
                    FROM
                        {$this->conf['table']['data_intermediate_ad_connection']} AS diac,
                        $trackerVariableValueTable AS drtvv,
                        $trackerImpressionTable AS drti
                    WHERE
                        diac.server_raw_tracker_impression_id = drti.server_raw_tracker_impression_id
                        AND diac.server_raw_ip = drti.server_raw_ip
                        AND drti.server_raw_tracker_impression_id = drtvv.server_raw_tracker_impression_id
                        AND drti.server_raw_ip = drtvv.server_raw_ip
                        AND diac.tracker_date_time >= '" . $start->format('%Y-%m-%d %H:%M:%S') . "'
                        AND diac.tracker_date_time <= '" . $end->format('%Y-%m-%d %H:%M:%S') . "'";
                debug("Saving the tracker impression variable values from the $innterTable table",
                      PEAR_LOG_DEBUG);
                $this->execute($query, $result);
                // Update the split data_raw_tracker_variable_value table being used
                $innerDate = $innerDate->getNextDay();
            }
            // Update the split data_raw_tracker_impression table being used
            $outerDate = $outerDate->getNextDay();
        }
    }
    
    /**
     * A method to delete old (ie. summarised) raw data.
     *
     * @param Date $summarisedTo The date/time up to which data have been summarised (i.e. if there
     *                           is no newer data than this date (minus any compact_stats_grace window)
     *                           then the raw table will be dropped, unless required by the tracking
     *                           module, where installed).
     * @return integer The number of raw tables dropped.
     */
    function deleteOldData($summarisedTo)
    {
        $deleteDate = $summarisedTo;
        if ($this->conf['maintenance']['compact_stats_grace'] > 0) {
            $deleteDate->subtractSeconds($this->conf['maintenance']['compact_stats_grace']);
        }
        if ($this->conf['modules']['Tracker']) {
            // Find the largest, active impression and click connection windows
            list($impressionWindow, $clickWindow) = Maintenance::maxConnectionWindows();
            // Find the largest of the two windows
            if ($impressionWindow > $clickWindow) {
                $maxWindow = $impressionWindow;
            } else {
                $maxWindow = $clickWindow;
            }
            debug('Found maximum connection window of ' . $maxWindow . ' seconds', PEAR_LOG_DEBUG);
            $deleteDate->subtractSeconds((int) $maxWindow); // Cast to int, as Date class
                                                            // doesn't deal with strings
        }
        $tables = 0;
        // As split tables are in use, look over all possible data_raw_ad_request tables
        $query = "SHOW TABLES like '{$this->conf['table']['data_raw_ad_request']}_%'";
        $this->execute($query, $result);
        if (mysql_num_rows($result) > 0) {
            while ($row = mysql_fetch_row($result)) {
                if (preg_match('/(\d{4})(\d{2})(\d{2})$/', $row[0], $matches)) {
                    $date = new Date("{$matches[1]}-{$matches[2]}-{$matches[3]}");
                    $table = $this->conf['table']['data_raw_ad_request'] . '_' . $date->format('%Y%m%d');
                    $query = "
                        SELECT
                            *
                        FROM
                            $table
                        WHERE
                            date_time > '" . $deleteDate->format('%Y-%m-%d %H:%M:%S') ."'";
                    debug("Selecting current (non-summarised) ad requests from the $table table", PEAR_LOG_DEBUG);
                    $errors = array(1146);   // Permit the table does not exist error
                    $errorID = $this->executeIgnoreErrors($query, $innerResult, $errors);
                    if (($errorID == 0) && (mysql_num_rows($innerResult) == 0)) {
                        // No current data in the table - drop it
                        $query = "DROP TABLE $table";
                        debug("No current (non-summarised) ad requests in the $table table, so dropping",
                              PEAR_LOG_DEBUG);
                        $this->execute($query, $innerResult);
                        $tables++;
                    }
                }
            }
        }
        // As split tables are in use, look over all possible data_raw_ad_impression tables
        $query = "SHOW TABLES like '{$this->conf['table']['data_raw_ad_impression']}_%'";
        $this->execute($query, $result);
        if (mysql_num_rows($result) > 0) {
            while ($row = mysql_fetch_row($result)) {
                if (preg_match('/(\d{4})(\d{2})(\d{2})$/', $row[0], $matches)) {
                    $date = new Date("{$matches[1]}-{$matches[2]}-{$matches[3]}");
                    $table = $this->conf['table']['data_raw_ad_impression'] . '_' . $date->format('%Y%m%d');
                    $query = "
                        SELECT
                            *
                        FROM
                            $table
                        WHERE
                            date_time > '" . $deleteDate->format('%Y-%m-%d %H:%M:%S') ."'";
                    debug("Selecting current (non-summarised) ad impressions from the $table table", PEAR_LOG_DEBUG);
                    $errors = array(1146);   // Permit the table does not exist error
                    $errorID = $this->executeIgnoreErrors($query, $innerResult, $errors);
                    if (($errorID == 0) && (mysql_num_rows($innerResult) == 0)) {
                        // No current data in the table - drop it
                        $query = "DROP TABLE $table";
                        debug("No current (non-summarised) ad impressions in the $table table, so dropping",
                              PEAR_LOG_DEBUG);
                        $this->execute($query, $innerResult);
                        $tables++;
                    }
                }
            }
        }
        // As split tables are in use, look over all possible data_raw_ad_click tables
        $query = "SHOW TABLES like '{$this->conf['table']['data_raw_ad_click']}_%'";
        $this->execute($query, $result);
        if (mysql_num_rows($result) > 0) {
            while ($row = mysql_fetch_row($result)) {
                if (preg_match('/(\d{4})(\d{2})(\d{2})$/', $row[0], $matches)) {
                    $date = new Date("{$matches[1]}-{$matches[2]}-{$matches[3]}");
                    $table = $this->conf['table']['data_raw_ad_click'] . '_' . $date->format('%Y%m%d');
                    $query = "
                        SELECT
                            *
                        FROM
                            $table
                        WHERE
                            date_time > '" . $deleteDate->format('%Y-%m-%d %H:%M:%S') ."'";
                    debug("Selecting current (non-summarised) ad clicks from the $table table", PEAR_LOG_DEBUG);
                    $errors = array(1146);   // Permit the table does not exist error
                    $errorID = $this->executeIgnoreErrors($query, $innerResult, $errors);
                    if (($errorID == 0) && (mysql_num_rows($innerResult) == 0)) {
                        // No current data in the table - drop it
                        $query = "DROP TABLE $table";
                        debug("No current (non-summarised) ad clicks in the $table table, so dropping",
                              PEAR_LOG_DEBUG);
                        $this->execute($query, $innerResult);
                        $tables++;
                    }
                }
            }
        }
        return $tables;
    }
    
}

?>