<?php

/*
+---------------------------------------------------------------------------+
| Max Media Manager v0.2                                                    |
| =================                                                         |
|                                                                           |
| Copyright (c) 2003-2004 Awarez Ltd.                                       |
| For contact details, see: http://www.awarez.net/                          |
|                                                                           |
| This program is free software; you can redistribute it and/or modify      |
| it under the terms of the GNU General Public License as published by      |
| the Free Software Foundation; either version 2 of the License, or         |
| (at your option) any later version.                                       |
|                                                                           |
| This program is distributed in the hope that it will be useful,           |
| but WITHOUT ANY WARRANTY; without even the implied warranty of            |
| MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             |
| GNU General Public License for more details.                              |
|                                                                           |
| You should have received a copy of the GNU General Public License         |
| along with this program; if not, write to the Free Software               |
| Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA |
+---------------------------------------------------------------------------+
$Id: StatisticsCommon.php 1018 2004-11-26 15:54:32Z andrew $
*/

/**
 * @package    MaxDal
 * @subpackage Statistics
 * @author     Andrew Hill <andrew@awarez.net>
 */

require_once maintenancePath . '/maintenance-common.php';
require_once maintenancePath . '/libraries/max/Dal.php';
require_once maintenancePath . '/libraries/max/Dal/TableCore.php';
require_once maintenancePath . '/libraries/max/Maintenance.php';
require_once maintenancePath . '/libraries/max/Maintenance/TableStatistics.php';

/**
 * An abstract class that defines the interface and some common methods for the data
 * access layer code for summarising raw data into statistics, for all Max modules.
 *
 * @abstract
 */
class Dal_Statistics_Common extends Dal
{
    var $tables;
    var $tempTables;
    
    /**
     * The constructor method.
     *
     * @uses Dal::Dal()
     */
    function Dal_Statistics_Common()
    {
        parent::Dal();
        $this->tables = Dal_Table_Core::singleton();
        $this->tempTables = Maintenance_Table_Statistics::singleton();
    }
    
    /**
     * The singleton method to create or return a single instance of the class.
     *
     * @static
     * @return Dal_Statistics_Common The returned Dal_Statistics_Common object.
     */
    function &singleton() {}

    /**
     * A method to find the last time that maintenance statistics was run.
     *
     * @param integer $type The update type that occurred - that is, 0 if
     *                      the update was done on the basis of the 
     *                      operation interval, or 1 if the update was
     *                      done on the basis of the hour.
     * @return Date A Date representing the date up to which the statistics
     *              have been summarised, for the specified update type, or
     *              the appropriate date based on raw data if maintenance
     *              statistics has never been run for the Max module before.
     *              Returns null if no raw data is available.
     */
    function getMaintenanceStatisticsLastRunInfo($type) {}
    
    /**
     * A method for summarising impressions into a temporary table.
     *
     * @param Date $start The start date/time to summarise from.
     * @param Date $end The end date/time to summarise to.
     * @return integer The number of conversion rows summarised.
     */
    function summariseImpressions($start, $end) {}

    /**
     * A method for summarising clicks into a temporary table.
     *
     * @param Date $start The start date/time to summarise from.
     * @param Date $end The end date/time to summarise to.
     * @return integer The number of conversion rows summarised.
     */    
    function summariseClicks($start, $end) {}
    
    /**
     * A method for summarising connections into a temporary table.
     *
     * @param Date $start The start date/time to summarise from.
     * @param Date $end The end date/time to summarise to.
     * @return integer The number of connections summarised.
     */
    function summariseConnections($start, $end) {}

    /**
     * A method to update the intermediate tables with summarised data.
     *
     * @param Date $start The start date/time to save from.
     * @param Date $end The end date/time to save to.
     */
    function saveIntermediate($start, $end) {}
    
    /**
     * A method to update the summary table from the intermediate tables.
     *
     * @param Date $from Update using data after this date.
     */
    function saveSummary($from) {}
    
    /**
     * A method to delete old (ie. summarised) raw data.
     *
     * @param Date $summarisedTo The date/time up to which data have been summarised (i.e. data up
     *                           to and including this date (minus any compact_stats_grace window)
     *                           will be deleted, unless required by the tracking module, where
     *                           installed).
     * @return integer The number of conversions deleted.
     */
    function deleteOldData($summarisedTo) {}
    
    /**
     * A method to store the details logging a maintenance statistics
     * run.
     *
     * @param Date $start The time that the maintenance statistics run
     *                    started.
     * @param Date $end The time that the maintenance statistics run
     *                  ended.
     * @param integer $type The type of run:
     *                          0 - The update was based on the operation interval;
     *                          1 - The update was based on the hour;
     *                          2 - The update was based on both the operation interval and hour.
     * @param Date $updateTo The end of the operation interval/hour that has been updated.
     * @param string $report The maintenance statistics run report.
     */
    function logCompletion($start, $end, $type, $updateTo, &$report) {}
    
    /**
     * A private method to check that two Dates represent either the
     * start and end of an operation interval, if the operation interval 
     * is less than an hour, or the start and end of an hour otherwise.
     *
     * @private
     * @param Date $start The interval start date.
     * @param Date $end The interval end date.
     * @return bool Returns true if the dates are correct interval
     *              start/end dates, false otherwise.
     */
    function _checkIntervalDates($start, $end)
    {    
        if ($this->conf['maintenance']['operation_interval'] <= 60) {
            // Must ensure that only one operation interval is being summarised
            $operationIntervalID = Maintenance::convertDateRangeToOperationIntervalID($start, $end);
            if (is_bool($operationIntervalID) && !$operationIntervalID) {
                return false;
            }
            // Now check that the start and end dates match the start and end 
            // of the operation interval
            list($operationIntervalStart, $operationIntervalEnd) = 
                Maintenance::convertDateToOperationIntervalStartAndEndDates($start);
            if (!$start->equals($operationIntervalStart)) {
                return false;
            }
            if (!$end->equals($operationIntervalEnd)) {
                return false;
            }
        } else {
            // Must ensure that only one hour is being summarised
            if (!Maintenance::checkDatesInSameHour($start, $end)) {
                return false;
            }
            // Now check that the start and end dates are match the start and
            // end of the hour
            $hourStart = new Date();
            $hourStart->setYear($start->getYear());
            $hourStart->setMonth($start->getMonth());
            $hourStart->setDay($start->getDay());
            $hourStart->setHour($start->getHour());
            $hourStart->setMinute('00');
            $hourStart->setSecond('00');
            $hourEnd = new Date();
            $hourEnd->setYear($end->getYear());
            $hourEnd->setMonth($end->getMonth());
            $hourEnd->setDay($end->getDay());
            $hourEnd->setHour($end->getHour());
            $hourEnd->setMinute('59');
            $hourEnd->setSecond('59');
            if (!$start->equals($hourStart)) {
                return false;
            }
            if (!$end->equals($hourEnd)) {
                return false;
            }
        }
        return true;
    }
    
}

?>