<?php

/*
+---------------------------------------------------------------------------+
| Max Media Manager v0.2                                                    |
| =================                                                         |
|                                                                           |
| Copyright (c) 2003-2004 Awarez Ltd.                                       |
| For contact details, see: http://www.awarez.net/                          |
|                                                                           |
| This program is free software; you can redistribute it and/or modify      |
| it under the terms of the GNU General Public License as published by      |
| the Free Software Foundation; either version 2 of the License, or         |
| (at your option) any later version.                                       |
|                                                                           |
| This program is distributed in the hope that it will be useful,           |
| but WITHOUT ANY WARRANTY; without even the implied warranty of            |
| MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             |
| GNU General Public License for more details.                              |
|                                                                           |
| You should have received a copy of the GNU General Public License         |
| along with this program; if not, write to the Free Software               |
| Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA |
+---------------------------------------------------------------------------+
$Id: TestOfDal.php 984 2004-11-24 17:38:45Z andrew $
*/

/**
 * @package    MaxDal
 * @subpackage TestSuite
 * @author     Andrew Hill <andrew@awarez.net>
 */

require_once maintenancePath . '/maintenance-common.php';
require_once maintenancePath . '/libraries/max/Dal.php';
require_once maintenancePath . '/libraries/pear/DB.php';
require_once maintenancePath . '/libraries/pear/Log.php';

/**
 * A class for testing the Dal class.
 */
class Dal_TestOfDal extends UnitTestCase
{
    
    /**
     * The constructor method.
     */
    function Dal_TestOfDal()
    {        
        $this->UnitTestCase();
    }
    
    /**
     * Tests that the database type is setup in the config .ini file.
     */
    function testDbTypeDefined()
    {
        debug('Running test', PEAR_LOG_INFO);
        $conf =& $GLOBALS['_MAX']['CONF'];
        $this->assertNotNull($conf['db']['type']);
        $this->assertNotEqual($conf['db']['type'], '');
    }
    
    /**
     * Tests that the database host is setup in the config .ini file.
     */
    function testDbHostDefined()
    {
        debug('Running test', PEAR_LOG_INFO);
        $conf =& $GLOBALS['_MAX']['CONF'];
        $this->assertNotNull($conf['db']['host']);
        $this->assertNotEqual($conf['db']['host'], '');
    }
    
    /**
     * Tests that the database port is setup in the config .ini file.
     */
    function testDbPortDefined()
    {
        debug('Running test', PEAR_LOG_INFO);
        $conf =& $GLOBALS['_MAX']['CONF'];
        $this->assertNotNull($conf['db']['port']);
        $this->assertNotEqual($conf['db']['port'], '');
    }
    
    /**
     * Tests that the database user is setup in the config .ini file.
     */
    function testDbUserDefined()
    {
        debug('Running test', PEAR_LOG_INFO);
        $conf =& $GLOBALS['_MAX']['CONF'];
        $this->assertNotNull($conf['db']['username']);
        $this->assertNotEqual($conf['db']['username'], '');
    }
    
    /**
     * Tests that the database password is setup in the config .ini file.
     */
    function testDbPasswordDefined()
    {
        debug('Running test', PEAR_LOG_INFO);
        $conf =& $GLOBALS['_MAX']['CONF'];
        $this->assertNotNull($conf['db']['password']);
        $this->assertNotEqual($conf['db']['password'], '');
    }
    
    /**
     * Tests that the database name is setup in the config .ini file.
     */
    function testDbNameDefined()
    {
        debug('Running test', PEAR_LOG_INFO);
        $conf =& $GLOBALS['_MAX']['CONF'];
        $this->assertNotNull($conf['db']['name']);
        $this->assertNotEqual($conf['db']['name'], '');
    }
    
    /**
     * Tests that the Max table prefix is setup in the config .ini file.
     */
    function testDbPrefixDefined()
    {
        debug('Running test', PEAR_LOG_INFO);
        $conf =& $GLOBALS['_MAX']['CONF'];
        $this->assertNotNull($conf['table']['prefix']);
        $this->assertNotEqual($conf['table']['prefix'], '');
    }
    
    /**
     * Tests that the database connection can be made, without using the
     * Dal class - that is, that the details specified above are okay.
     */
    function testDbConnection()
    {
        debug('Running test', PEAR_LOG_INFO);
        $conf =& $GLOBALS['_MAX']['CONF'];
        $dbConnection = DB::connect($conf['db']['type'] . '://' . 
                        $conf['db']['username'] . ':' .  $conf['db']['password'] . 
                        '@' . $conf['db']['host'] . ':' . $conf['db']['port'] . '/' .
                        $conf['db']['name']);
        $this->assertTrue($dbConnection);
    }

    /**
     * Tests that the singleton() method only ever returns one
     * database connection.
     */
    function testSingletonDbConnection()
    {
        debug('Running test', PEAR_LOG_INFO);
        $conf =& $GLOBALS['_MAX']['CONF'];
        $backup = $conf['db']['name'];
        $conf['db']['name'] = 'test';
        $dal =& Dal::singleton();
        $firstConnection = $dal->dbConnection;
        $dal =& Dal::singleton();
        $secondConnection = $dal->dbConnection;
        $this->assertIdentical($firstConnection, $secondConnection);
        $conf['db']['name'] = $backup;
    }
    
}

?>