<?php

/*
+---------------------------------------------------------------------------+
| Max Media Manager v0.2                                                    |
| =================                                                         |
|                                                                           |
| Copyright (c) 2003-2004 Awarez Ltd.                                       |
| For contact details, see: http://www.awarez.net/                          |
|                                                                           |
| This program is free software; you can redistribute it and/or modify      |
| it under the terms of the GNU General Public License as published by      |
| the Free Software Foundation; either version 2 of the License, or         |
| (at your option) any later version.                                       |
|                                                                           |
| This program is distributed in the hope that it will be useful,           |
| but WITHOUT ANY WARRANTY; without even the implied warranty of            |
| MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             |
| GNU General Public License for more details.                              |
|                                                                           |
| You should have received a copy of the GNU General Public License         |
| along with this program; if not, write to the Free Software               |
| Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA |
+---------------------------------------------------------------------------+
$Id: TestOfStatisticsCommon.php 1008 2004-11-26 10:52:27Z andrew $
*/

/**
 * @package    MaxDal
 * @subpackage TestSuite
 * @author     Andrew Hill <andrew@awarez.net>
 */

require_once maintenancePath . '/maintenance-common.php';
require_once maintenancePath . '/libraries/max/Dal/StatisticsCommon.php';
require_once maintenancePath . '/libraries/pear/Date.php';
require_once maintenancePath . '/libraries/pear/Log.php';

/**
 * A class for testing the Dal_Statistics_Common class.
 */
class Dal_TestOfStatisticsCommon extends UnitTestCase
{
    
    /**
     * The constructor method.
     */
    function Dal_TestOfStatisticsCommon()
    {        
        $this->UnitTestCase();
    }
    
    /**
     * Tests the constructor method.
     */
    function testConstructor()
    {
        debug('Running test', PEAR_LOG_INFO);
        $dsc = new Dal_Statistics_Common();
        // Only test the constructor code specific to the Dal_Statistics_Common class
        $this->assertEqual(get_class($dsc->tables), 'dal_table_core');
        $this->assertEqual(get_class($dsc->tempTables), 'maintenance_table_statistics');
    }
    
    /**
     * Tests the private _checkIntervalDates() method.
     */
    function testCheckIntervalDates()
    {
        debug('Running test', PEAR_LOG_INFO);
        $conf =& $GLOBALS['_MAX']['CONF'];
        // Create a Dal_Statistics_Common object
        $dsc = new Dal_Statistics_Common();
        // Set the operation interval
        $conf['maintenance']['operation_interval'] = 30;
        // Test less than one operation interval
        $start = new Date('2004-09-26 00:00:00');
        $end = new Date('2004-09-26 00:15:00');
        $this->assertFalse($dsc->_checkIntervalDates($start, $end));
        // Test more than one operation inteterval
        $start = new Date('2004-09-26 00:00:00');
        $end = new Date('2004-09-26 00:45:00');
        $this->assertFalse(Dal_Statistics_Common::_checkIntervalDates($start, $end));
        // Test exactly one operation interval
        $start = new Date('2004-09-26 00:00:00');
        $end = new Date('2004-09-26 00:29:59');
        $this->assertTrue($dsc->_checkIntervalDates($start, $end));
        // Set the operation interval
        $conf['maintenance']['operation_interval'] = 60;
        // Test less than one operation interval/hour
        $start = new Date('2004-09-26 00:00:00');
        $end = new Date('2004-09-26 00:30:00');
        $this->assertFalse($dsc->_checkIntervalDates($start, $end));
        // Test more than one operation inteterval/hour
        $start = new Date('2004-09-26 00:00:00');
        $end = new Date('2004-09-26 01:15:00');
        $this->assertFalse(Dal_Statistics_Common::_checkIntervalDates($start, $end));
        // Test exactly one operation interval/hour
        $start = new Date('2004-09-26 00:00:00');
        $end = new Date('2004-09-26 00:59:59');
        $this->assertTrue($dsc->_checkIntervalDates($start, $end));
        // Set the operation interval
        $conf['maintenance']['operation_interval'] = 120;
        // Test less than one hour
        $start = new Date('2004-09-26 00:00:00');
        $end = new Date('2004-09-26 00:15:00');
        $this->assertFalse($dsc->_checkIntervalDates($start, $end));
        // Test more than one hour
        $start = new Date('2004-09-26 00:00:00');
        $end = new Date('2004-09-26 04:00:00');
        $this->assertFalse(Dal_Statistics_Common::_checkIntervalDates($start, $end));
        // Test exactly one hour
        $start = new Date('2004-09-26 00:00:00');
        $end = new Date('2004-09-26 00:59:59');
        $this->assertTrue($dsc->_checkIntervalDates($start, $end));
    }
    
}

?>